% File src/library/base/man/detach.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{detach}
\alias{detach}
\title{Detach Objects from the Search Path}
\usage{
detach(name, pos = 2L, unload = FALSE, character.only = FALSE,
       force = FALSE)
}
\arguments{
  \item{name}{
    登録を解除するオブジェクト．既定では \code{search()[pos]}．
    これは引用化されていない名前でも，文字列であるが文字列ベクトルではないものでも良い．
    もし数字が与えられればそれは \code{pos} とされる．
  }
  \item{pos}{
    登録解除するデータベースの \code{\link{search}()} 中の位置添字．
    \code{name} が数字であれば \code{pos = name} が使われる．
  }
  \item{unload}{パッケージを登録解除する際名前空間を取り除くことを試みるかどうかを
    指示する論理値．もしパッケージが名前空間を持ち \code{unload} が \code{TRUE} ならば
    \code{detach} は \code{\link{unloadNamespace}} を使って名前空間を取り除こうとする：
    もし名前空間が別の名前空間から移入されているか，または \code{unload} が \code{FALSE}
    ならば名前空間は取り除かれない．
  }
  \item{character.only}{\code{name} が文字列であると仮定しても良いかどうかを
    指示する論理値．}
  \item{force}{論理値：他の付加されたパッケージが依存しているパッケージを
    除いても良いか?}
}
\description{
  データベースの登録を解除する，つまり，それを利用可能な \R オブジェクトの検索パスから
  取り除く．普通これは \code{\link{attach}} 関数で付加されている
  \code{\link{data.frame}} か \code{\link{library}} 関数で付加された
  パッケージである．
}
\details{
  これは最も普通に検索リスト上の位置を表す単一の数字と共に使われ，
  \code{package:tools} のような引用化されたりされていない項目の名前
  とともに使うことも出来る．

  もしパッケージが名前空間を持てば，それの登録解除は既定では
  名前空間を除去しない(そして \code{unload = TRUE} であったとしても)．
  そして登録解除は一般に動的に読み込まれたコンパイル済みコード(DLL)も
  除去しない．更に，名前空間からの登録された S3 メソッドも取り除かれない．
  その名前空間が読み込まれたパッケージに対して \code{\link{library}} を
  使うと，それは既に読み込まれている名前空間を移出したものを付加する．
  従ってパッケージの登録解除と再付加はパッケージ中の幾つかかすべての成分を
  更新しないかもしれず，推奨できない．
}
\section{良い作法}{
  引数無しの \code{detach()} は検索パス上の作業空間の後の最初の項目を
  取り去る．それを何度もまたは僅かな回数呼び出すのも極めて簡単で，
  \code{\link{attach}} の呼び出し後に検索パスが変化していることを
  注意する必要もない．
  
  \code{attach}/\code{detach} の関数中での使用は極力
  避けたほうが良い(\code{\link{attach}} に対するヘルプを見よ)．
  そして対話的な使用とスクリプト中では名前で取り除くのが賢明である．
}
\note{
  作業空間(位置 1) も \pkg{base} パッケージ(検索リストの最後の項目)も
  取り除くことはできない．もしそうしようとするとエラーになる．

  ある種の名前空間の登録解除は望ましくない副作用を持つ：
  例えば \pkg{grid} を除けばすべてのグラフィックデバイスが閉じられる．
  そしてある種のシステムでは \pkg{tcltk} は一度取り除かれると再登録できず，
  再登録を試みると \R はクラッシュするかもしれない．
}
\value{
  返り値は不可視である．パッケージが除かれると \code{NULL} で，
  さもなければオブジェクトが付加された時に \code{\link{attach}} が返した
  環境(それが付加されて以来の変化を取り入れた)である．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  \code{\link{attach}}, \code{\link{library}}, \code{\link{search}},
  \code{\link{objects}}, \code{\link{unloadNamespace}},
  \code{\link{library.dynam.unload}} .
}
\examples{
require(splines) # パッケージ
detach(package:splines)
## 又は
library(splines)
pkg <- "package:splines"
\dontshow{
stopifnot(inherits(tryCatch(detach(pkg), error = function(.).),  "error"))
}
detach(pkg, character.only = TRUE)

## 注意深く：'splines' が既に付加されていない限りこれをしてはならない
library(splines)
detach(2) # 名前の代わりに位置を使う

## atatch への名前引数と
## 文字列ベクトルで名前が付けられたデータベースを取り除く例
attach_and_detach <- function(db, pos = 2)
{
   name <- deparse(substitute(db))
   attach(db, pos = pos, name = name)
   print(search()[pos])
   detach(name, character.only = TRUE)
}
attach_and_detach(women, pos = 3)
}
\keyword{data}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/detach.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{detach}
%\alias{detach}
%\title{Detach Objects from the Search Path}
%\usage{
%detach(name, pos = 2L, unload = FALSE, character.only = FALSE,
%       force = FALSE)
%}
%\arguments{
%  \item{name}{
%    The object to detach.  Defaults to \code{search()[pos]}.
%    This can be an unquoted name or a character string but \emph{not} a
%    character vector.  If a number is supplied this is taken as \code{pos}.
%  }
%  \item{pos}{
%    Index position in \code{\link{search}()} of the database to
%    detach.  When \code{name} is a number, \code{pos = name}
%    is used.
%  }
%  \item{unload}{A logical value indicating whether or not to attempt to
%    unload the namespace when a package is being detached.  If the
%    package has a namespace and \code{unload} is \code{TRUE}, then
%    \code{detach} will attempt to unload the namespace \emph{via}
%    \code{\link{unloadNamespace}}: if the namespace is imported by
%    another namespace or \code{unload} is \code{FALSE}, no unloading
%    will occur.
%  }
%  \item{character.only}{a logical indicating whether \code{name}
%    can be assumed to be a character string.}
%  \item{force}{logical: should a package be detached even though other
%    attached packages depend on it?}
%}
%\description{
%  Detach a database, i.e., remove it from the \code{\link{search}()}
%  path of available \R objects.  Usually this is either a
%  \code{\link{data.frame}} which has been \code{\link{attach}}ed or a
%  package which was attached by \code{\link{library}}.
%}
%\details{
%  This is most commonly used with a single number argument referring to a
%  position on the search list, and can also be used with a unquoted or
%  quoted name of an item on the search list such as \code{package:tools}.
%
%  If a package has a namespace, detaching it does not by default unload
%  the namespace (and may not even with \code{unload = TRUE}), and
%  detaching will not in general unload any dynamically loaded compiled
%  code (DLLs).  Further, registered S3 methods from the namespace will
%  not be removed.  If you use \code{\link{library}} on a package whose
%  namespace is loaded, it attaches the exports of the already loaded
%  namespace.  So detaching and re-attaching a package may not refresh
%  some or all components of the package, and is inadvisable.
%}
%\section{Good practice}{
%  \code{detach()} without an argument removes the first item on the
%  search path after the workspace.  It is all too easy to call it too
%  many or too few times, or to not notice that the search path has
%  changed since an \code{\link{attach}} call.
%
%  Use of \code{attach}/\code{detach} is best avoided in functions (see
%  the help for \code{\link{attach}}) and in interactive use and scripts
%  it is prudent to detach by name.
%}
%\note{
%  You cannot detach either the workspace (position 1) nor the \pkg{base}
%  package (the last item in the search list), and attempting to do so
%  will throw an error.
%
%  Unloading some namespaces has undesirable side effects:
%  e.g.\sspace{}unloading \pkg{grid} closes all graphics devices, and on some
%  systems \pkg{tcltk} cannot be reloaded once it has been unloaded and
%  may crash \R if this is attempted.
%}
%\value{
%  The return value is \link{invisible}.  It is \code{NULL} when a
%  package is detached, otherwise the environment which was returned by
%  \code{\link{attach}} when the object was attached (incorporating any
%  changes since it was attached).
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{attach}}, \code{\link{library}}, \code{\link{search}},
%  \code{\link{objects}}, \code{\link{unloadNamespace}},
%  \code{\link{library.dynam.unload}} .
%}
%\examples{
%require(splines) # package
%detach(package:splines)
%## or also
%library(splines)
%pkg <- "package:splines"
%\dontshow{
%stopifnot(inherits(tryCatch(detach(pkg), error = function(.).),  "error"))
%}
%detach(pkg, character.only = TRUE)
%
%## careful: do not do this unless 'splines' is not already attached.
%library(splines)
%detach(2) # 'pos' used for 'name'
%
%## an example of the name argument to attach
%## and of detaching a database named by a character vector
%attach_and_detach <- function(db, pos = 2)
%{
%   name <- deparse(substitute(db))
%   attach(db, pos = pos, name = name)
%   print(search()[pos])
%   detach(name, character.only = TRUE)
%}
%attach_and_detach(women, pos = 3)
%}
%\keyword{data}
%
\encoding{UTF8}
