% File src/library/base/man/solve.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{solve}
\title{Solve a System of Equations}
\description{
  この総称的関数は方程式 \code{a \%*\% x = b} を \code{x} について解く．
  ここで \code{b} はベクトルでも行列でも良い．
}
\usage{
solve(a, b, \dots)

\method{solve}{default}(a, b, tol, LINPACK = FALSE, \dots)
}
\alias{solve}
\alias{solve.default}
\arguments{
  \item{a}{正方数値もしくは複素数行列で線形システムの係数を含む．
    論理値行列は数値に変換される．}
  \item{b}{線形システムの右辺値を与える数値もしくは複素数ベクトル又は行列．
    もし欠損していれば \code{b} は単位行列で \code{solve} は
    \code{a} の逆行列を返す．}
  \item{tol}{\code{a} の列の間の線形従属性を検出するための許容度で
    \code{.Machine$double.eps} が既定値．現在複素行列 
    \code{a} に対しては使われない．}
  \item{LINPACK}{論理値．廃止され無視される．}
  \item{\dots}{他のメソッドに・から渡される追加引数．}
}
\details{
  \code{a} 又は \code{b} は複素数値であって良いが，これは倍精度複素算術を
  使っており全てのプラットフォームで利用できるとは限らない．

  結果の行と列名は
  それぞれ \code{a} と/又は \code{b} の列名から取られる．
  もし \code{b} が無ければ結果の列名は \code{a} の行名である．
  \code{a} の列名が \code{b} の行名が等しいかどうかのチェックはされない．

  過去との互換性から \code{a} は(実) QR 分解であって良いが，
  その場合は \code{\link{qr.solve}} が呼ばれるべきである．
  \code{\link{qr.solve}} は非正方システムを扱える．

  基礎にある LAPACK コードからの不成功の結果は正のエラーコードを与える
  エラーになる：これらは FORTRAN コードの詳細な吟味だけによって解釈できる．
}
\source{
  既定のメソッドは LAPACK ルーチン \code{DGESV} と \code{ZGESV} へのインタフェイスである．

  LAPACK は \url{http://www.netlib.org/lapack} にある．
}
\references{
  Anderson. E. and ten others (1999)
  \emph{LAPACK Users' Guide}. Third Edition. SIAM.\cr
  Available on-line at
  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.

  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}

\seealso{
  \code{qr} メソッドについては \code{\link{solve.qr}}．
  Choleski 因子からの逆行列に付いては \code{\link{chol2inv}}．
  \code{\link{backsolve}}, \code{\link{qr.solve}}.
}
\examples{
hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
h8 <- hilbert(8); h8
sh8 <- solve(h8)
round(sh8 \%*\% h8, 3)

A <- hilbert(4)
A[] <- as.complex(A)
## 全てのプラットフォームではサポートされていないかもしれない
try(solve(A))
}
\keyword{algebra}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/solve.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{solve}
%\title{Solve a System of Equations}
%\description{
%  This generic function solves the equation \code{a \%*\% x = b} for \code{x},
%  where \code{b} can be either a vector or a matrix.
%}
%\usage{
%solve(a, b, \dots)
%
%\method{solve}{default}(a, b, tol, LINPACK = FALSE, \dots)
%}
%\alias{solve}
%\alias{solve.default}
%\arguments{
%  \item{a}{a square numeric or complex matrix containing the coefficients of
%    the linear system.  Logical matrices are coerced to numeric.}
%  \item{b}{a numeric or complex vector or matrix giving the right-hand
%    side(s) of the linear system.  If missing, \code{b} is taken to be
%    an identity matrix and \code{solve} will return the inverse of \code{a}.}
%  \item{tol}{the tolerance for detecting linear dependencies in the
%    columns of \code{a}.  The default is \code{.Machine$double.eps}. Not
%    currently used with complex matrices \code{a}.}
%  \item{LINPACK}{logical.  Defunct and ignored.}
%  \item{\dots}{further arguments passed to or from other methods}
%}
%\details{
%  \code{a} or \code{b} can be complex, but this uses double complex
%  arithmetic which might not be available on all platforms.
%
%  The row and column names of the result are taken from the column names
%  of \code{a} and of \code{b} respectively.  If \code{b} is missing the
%  column names of the result are the row names of \code{a}.  No check is
%  made that the column names of \code{a} and the row names of \code{b}
%  are equal.
%
%  For back-compatibility \code{a} can be a (real) QR decomposition,
%  although \code{\link{qr.solve}} should be called in that case.
%  \code{\link{qr.solve}} can handle non-square systems.
%
%  Unsuccessful results from the underlying LAPACK code will result in an
%  error giving a positive error code: these can only be interpreted by
%  detailed study of the FORTRAN code.
%}
%\source{
%  The default method is an interface to the LAPACK routines \code{DGESV}
%  and \code{ZGESV}.
%
%  LAPACK is from \url{http://www.netlib.org/lapack}.
%}
%\references{
%  Anderson. E. and ten others (1999)
%  \emph{LAPACK Users' Guide}. Third Edition. SIAM.\cr
%  Available on-line at
%  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.
%
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%
%\seealso{
%  \code{\link{solve.qr}} for the \code{qr} method,
%  \code{\link{chol2inv}} for inverting from the Choleski factor
%  \code{\link{backsolve}}, \code{\link{qr.solve}}.
%}
%\examples{
%hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
%h8 <- hilbert(8); h8
%sh8 <- solve(h8)
%round(sh8 \%*\% h8, 3)
%
%A <- hilbert(4)
%A[] <- as.complex(A)
%## might not be supported on all platforms
%try(solve(A))
%}
%\keyword{algebra}
%
\encoding{UTF8}
