% File src/library/base/man/svd.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{svd}
\alias{svd}
\alias{La.svd}
\title{Singular Value Decomposition of a Matrix}
\description{
  矩形行列の特異値分解を計算する．
}
\usage{
svd(x, nu = min(n, p), nv = min(n, p), LINPACK = FALSE)

La.svd(x, nu = min(n, p), nv = min(n, p))
}
\arguments{
  \item{x}{SVD 分解を計算すべき数値もしくは複素数値行列．
    論理値行列は数値に変換される．}
  \item{nu}{計算されるべき左特異ベクトルの数．
    これは \code{0} と \code{n = nrow(x)} の間になければならない．}
  \item{nv}{計算されるべき右特異ベクトルの数．
    これは \code{0} と \code{n = ncol(x)} の間になければならない．}
  \item{LINPACK}{論理値．廃止され無視される．}
}
\details{
  特異値分解は多くの統計的技術中で重要な役割を果たす．
  \code{svd} と \code{La.svd} はそれらの返り値に於いて異なる
  二つのインタフェイスを提供する．

  特異ベクトルの計算は大きな行列に対しては時間のかかる部分である．
  計算はもし \code{nu <= min(n, p)} で \code{nv <= min(n, p)} の時
  最も効率的で，もし共にゼロなら更に効率的である．

  基礎にある LAPACK コードからの不成功の結果は正のエラーコード
  (最も普通には \code{1})をもつエラーを与える：これらは FORTRAN コードの
  詳細な吟味によってのみ解釈できるがアルゴリズムが収束しなかったことを意味する．
}
\value{
  LAPACK により計算された行列の SVD 分解 \deqn{\bold{X = U D V'},}
  ここで \eqn{\bold{U}} と \eqn{\bold{V}} は直交で，
  \eqn{\bold{V'}} は\emph{V の転置} (そして複素数入力に対しては共役)を
  意味し，\eqn{\bold{D}} は特異値 \eqn{D_{ii}}{D[i,i]} の対角行列
  である．同値であるが \eqn{\bold{D = U' X V}} でもあり．
  例の中で検証される．

  返り値は次の成分を持つリストを返す
  \item{d}{\code{x} の特異値を含むベクトルで長さ \code{min(n, p)}．}
  \item{u}{その列が \code{x} の左特異ベクトルを含む行列で，
    もし \code{nu > 0} なら存在する．次元は \code{c(n, nu)}．}
  \item{v}{その列が \code{x} の右特異ベクトルを含む行列で，
    もし \code{nv > 0} なら存在する．次元は \code{c(p, nv)}．}

  特異ベクトル符号(複素数ケースでは絶対値 1 の定数)を除いてのみ定義される
  ことを思い出そう．もし左特異ベクトルがその符号を変更すれば，対応する
  右ベクトルが同値な分解を与える．

  \code{La.svd} に対しては返り値は \code{v} をその転置 \code{vt}
  (もし複素数なら共役)で置き換えたものになる．
}
\source{
  使用される主要関数は LAPACK ルーチン \code{DGESDD} と
  \code{ZGESDD} である．

  LAPACK は \url{http://www.netlib.org/lapack} から，そしてそのガイドは
  参考文献にある．
}
\references{
  Anderson. E. and ten others (1999)
  \emph{LAPACK Users' Guide}. Third Edition. SIAM.\cr
  Available on-line at
  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.

  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}

\seealso{
  \code{\link{eigen}}, \code{\link{qr}}.
}
\examples{
hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
X <- hilbert(9)[, 1:6]
(s <- svd(X))
D <- diag(s$d)
s$u \%*\% D \%*\% t(s$v) #  X = U D V'
t(s$u) \%*\% X \%*\% s$v #  D = U' X V
}
\keyword{algebra}
\keyword{array}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/svd.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{svd}
%\alias{svd}
%\alias{La.svd}
%\title{Singular Value Decomposition of a Matrix}
%\description{
%  Compute the singular-value decomposition of a rectangular matrix.
%}
%\usage{
%svd(x, nu = min(n, p), nv = min(n, p), LINPACK = FALSE)
%
%La.svd(x, nu = min(n, p), nv = min(n, p))
%}
%\arguments{
%  \item{x}{a numeric or complex matrix whose SVD decomposition
%    is to be computed.  Logical matrices are coerced to numeric.}
%  \item{nu}{the number of left  singular vectors to be computed.
%    This must between \code{0} and \code{n = nrow(x)}.}
%  \item{nv}{the number of right singular vectors to be computed.
%    This must be between \code{0} and \code{p = ncol(x)}.}
%  \item{LINPACK}{logical.  Defunct and ignored.}
%}
%\details{
%  The singular value decomposition plays an important role in many
%  statistical techniques.  \code{svd} and \code{La.svd} provide two
%  interfaces which differ in their return values.
%
%  Computing the singular vectors is the slow part for large matrices.
%  The computation will be more efficient if both \code{nu <= min(n, p)}
%  and \code{nv <= min(n, p)}, and even more so if both are zero.
%
%  Unsuccessful results from the underlying LAPACK code will result in an
%  error giving a positive error code (most often \code{1}): these can
%  only be interpreted by detailed study of the FORTRAN code but mean
%  that the algorithm failed to converge.
%}
%\value{
%  The SVD decomposition of the matrix as computed by LAPACK, \deqn{
%  \bold{X = U D V'},} where \eqn{\bold{U}} and \eqn{\bold{V}} are
%  orthogonal, \eqn{\bold{V'}} means \emph{V transposed} (and conjugated
%  for complex input), and \eqn{\bold{D}} is a diagonal matrix with the
%  singular values \eqn{D_{ii}}{D[i,i]}.  Equivalently, \eqn{\bold{D = U'
%  X V}}, which is verified in the examples.
%
%  The returned value is a list with components
%  \item{d}{a vector containing the singular values of \code{x}, of
%    length \code{min(n, p)}.}
%  \item{u}{a matrix whose columns contain the left singular vectors of
%    \code{x}, present if \code{nu > 0}.  Dimension \code{c(n, nu)}.}
%  \item{v}{a matrix whose columns contain the right singular vectors of
%    \code{x}, present if \code{nv > 0}.  Dimension \code{c(p, nv)}.}
%
%  Recall that the singular vectors are only defined up to sign (a
%  constant of modulus one in the complex case).  If a left singular
%  vector has its sign changed, changing the sign of the corresponding
%  right vector gives an equivalent decomposition.
%
%  For \code{La.svd} the return value replaces \code{v} by \code{vt}, the
%  (conjugated if complex) transpose of \code{v}.
%}
%\source{
%  The main functions used are the LAPACK routines \code{DGESDD} and
%  \code{ZGESDD}.
%
%  LAPACK is from \url{http://www.netlib.org/lapack} and its guide is
%  listed in the references.
%}
%\references{
%  Anderson. E. and ten others (1999)
%  \emph{LAPACK Users' Guide}. Third Edition. SIAM.\cr
%  Available on-line at
%  \url{http://www.netlib.org/lapack/lug/lapack_lug.html}.
%
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%
%\seealso{
%  \code{\link{eigen}}, \code{\link{qr}}.
%}
%\examples{
%hilbert <- function(n) { i <- 1:n; 1 / outer(i - 1, i, "+") }
%X <- hilbert(9)[, 1:6]
%(s <- svd(X))
%D <- diag(s$d)
%s$u \%*\% D \%*\% t(s$v) #  X = U D V'
%t(s$u) \%*\% X \%*\% s$v #  D = U' X V
%}
%\keyword{algebra}
%\keyword{array}
%
%