% File src/library/base/man/switch.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{switch}
\alias{switch}
\title{Select One of a List of Alternatives}
\description{
  \code{switch} は \code{EXPR} を評価し，それに応じて(\code{\dots} 中の)
  追加引数の一つを選択する．
}
\usage{
switch(EXPR, \dots)
}
\arguments{
  \item{EXPR}{評価結果が数値か文字列になる表現式．}
  \item{\dots}{選択肢のリスト．もし
    \code{EXPR} が文字列値を持つことを意図しているならば，
    これらは名前を持つ．例外はもしかすると
    \sQuote{既定}値として使われる一つの選択肢である．}
}
\details{
  \code{switch} は最初の引数が文字列になるか数値になるかに応じて
  二つの仕方で動作する．

  もし \code{EXPR} の値が文字列でなければそれは整数に変換される．
  これはまた \code{\link{factor}} に対しても警告付きで起こり，
  典型的に文字列水準が意味されている．もし整数が
  1 と \code{nargs()-1} の間にあれば対応する
  \code{\dots} の要素が評価され結果が返される：このようにしてもし
  最初の引数が \code{3} ならば4番目の引数が評価され返される．

  もし \code{EXPR} の評価結果が文字列ならば，その文字列は
  \code{\dots} 中の要素の名前に(正確に)マッチする．
  もしマッチがあればその要素が欠損していない限り評価される．
  欠損している場合は次の非欠損要素が評価される．
  従って例えば \code{switch("cc", a = 1, cc =, cd =, d = 2)} の評価結果は
  \code{2} になる．もしひとつ以上のマッチがあれば最初のマッチ要素が使われる．
  マッチするものが皆無ならば，もし \code{\dots} 中に名前のない要素があれば
  その値が使われる．(もしそうした引数が一つ以上あればエラーが返される．)

  最初の引数は常に \code{EXPR} とされる：もしそれが名前を持てば
  その名前が(部分的に)マッチしなければならない．

  これはプリミティブ関数として実装され，
  その最初の引数だけを評価し，もし一つが選択されたならば他の一つを評価する．
}
\section{警告}{
  以前のバージョンの \R では混乱を招き同じようには動作しない \code{switch} への
  呼び出しを書くことが可能であった．
  互換性(と明快さ)のためには，常に \code{EXPR} を最初の引数にし，
  もし部分マッチの可能性があればそれに名前を付ける．
  文字列形式では，名前付き値の後に既定値として名前無しの引数を持つ．
}
\value{
  \code{\dots} の要素の一つの値，又は(どの要素も選択されなければ)
  不可視の \code{NULL}．

  結果は評価された要素の可視性を持つ(\code{\link{invisible}} を見よ)．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\examples{
require(stats)
centre <- function(x, type) {
  switch(type,
         mean = mean(x),
         median = median(x),
         trimmed = mean(x, trim = .1))
}
x <- rcauchy(10)
centre(x, "mean")
centre(x, "median")
centre(x, "trimmed")

ccc <- c("b","QQ","a","A","bb")
# 注意：cat() は NULL に対して出力を生成しない
for(ch in ccc)
    cat(ch,":", switch(EXPR = ch, a = 1, b = 2:3), "\n")
for(ch in ccc)
    cat(ch,":", switch(EXPR = ch, a =, A = 1, b = 2:3, "Otherwise: last"),"\n")

## 因子  f で switch(f, *)
ff <- gl(3,1, labels=LETTERS[3:1])
ff[1] # C
## 従ってここで " is C" を期待するが，しかし
switch(ff[1], A = "I am A", B="Bb..", C=" is C")# -> "A"
## 従って警告を得る

## 数値の EXPR は既定値の指定を認めない
## -- これは常に NULL
for(i in c(-1:3, 9))  print(switch(i, 1, 2 , 3, 4))

## 可視性
switch(1, invisible(pi), pi)
switch(2, invisible(pi), pi)
}
\keyword{programming}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/switch.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{switch}
%\alias{switch}
%\title{Select One of a List of Alternatives}
%\description{
%  \code{switch} evaluates \code{EXPR} and accordingly chooses one of the
%  further arguments (in \code{\dots}).
%}
%\usage{
%switch(EXPR, \dots)
%}
%\arguments{
%  \item{EXPR}{an expression evaluating to a number or a character
%    string.}
%  \item{\dots}{the list of alternatives.  If it is intended that
%    \code{EXPR} has a character-string value these will be
%    named, perhaps except for one alternative to be used as a
%    \sQuote{default} value.}
%}
%\details{
%  \code{switch} works in two distinct ways depending whether the first
%  argument evaluates to a character string or a number.
%
%  If the value of \code{EXPR} is not a character string it is coerced to
%  integer.  Note that this also happens for \code{\link{factor}}s, with
%  a warning, as typically the character level is meant.  If the integer
%  is between 1 and \code{nargs()-1} then the corresponding element of
%  \code{\dots} is evaluated and the result returned: thus if the first
%  argument is \code{3} then the fourth argument is evaluated and
%  returned.
%
%  If \code{EXPR} evaluates to a character string then that string is
%  matched (exactly) to the names of the elements in \code{\dots}.  If
%  there is a match then that element is evaluated unless it is missing,
%  in which case the next non-missing element is evaluated, so for
%  example \code{switch("cc", a = 1, cc =, cd =, d = 2)} evaluates to
%  \code{2}.  If there is more than one match, the first matching element
%  is used.  In the case of no match, if there is a unnamed element of
%  \code{\dots} its value is returned.  (If there is more than one such
%  argument an error is returned.)
%
%  The first argument is always taken to be \code{EXPR}: if it is named
%  its name must (partially) match.
%
%  This is implemented as a \link{primitive} function that only evaluates
%  its first argument and one other if one is selected.
%}
%\section{Warning}{
%  It is possible to write calls to \code{switch} that can be confusing
%  and may not work in the same way in earlier versions of \R.  For
%  compatibility (and clarity), always have \code{EXPR} as the first
%  argument, naming it if partial matching is a possibility.  For the
%  character-string form, have a single unnamed argument as the default
%  after the named values.
%}
%\value{
%  The value of one of the elements of \code{\dots}, or \code{NULL},
%  invisibly (whenever no element is selected).
%
%  The result has the visibility (see \code{\link{invisible}}) of the
%  element evaluated.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\examples{
%require(stats)
%centre <- function(x, type) {
%  switch(type,
%         mean = mean(x),
%         median = median(x),
%         trimmed = mean(x, trim = .1))
%}
%x <- rcauchy(10)
%centre(x, "mean")
%centre(x, "median")
%centre(x, "trimmed")
%
%ccc <- c("b","QQ","a","A","bb")
%# note: cat() produces no output for NULL
%for(ch in ccc)
%    cat(ch,":", switch(EXPR = ch, a = 1, b = 2:3), "\n")
%for(ch in ccc)
%    cat(ch,":", switch(EXPR = ch, a =, A = 1, b = 2:3, "Otherwise: last"),"\n")
%
%## switch(f, *) with a factor f
%ff <- gl(3,1, labels=LETTERS[3:1])
%ff[1] # C
%## so one might expect " is C" here, but
%switch(ff[1], A = "I am A", B="Bb..", C=" is C")# -> "A"
%## so we give a warning
%
%## Numeric EXPR does not allow a default value to be specified
%## -- it is always NULL
%for(i in c(-1:3, 9))  print(switch(i, 1, 2 , 3, 4))
%
%## visibility
%switch(1, invisible(pi), pi)
%switch(2, invisible(pi), pi)
%}
%\keyword{programming}
%