% File src/library/base/man/userhooks.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{userhooks}
\alias{getHook}
\alias{setHook}
\alias{packageEvent}
\alias{.userHooksEnv}
\title{Functions to Get and Set Hooks for Load, Attach, Detach and Unload}
\description{
  これらの関数はパッケージが付加/除去されたり，名前空間が(アン)ロードされる
  前に取られるアクションをユーザが設定出来るようにする．
}
\usage{
getHook(hookName)
setHook(hookName, value,
        action = c("append", "prepend", "replace"))

packageEvent(pkgname,
             event = c("onLoad", "attach", "detach", "onUnload"))
}
\arguments{
  \item{hookName}{文字列：フック名．}
  \item{pkgname}{文字列：パッケージ/名前空間の名前．}
  \item{event}{文字列：パッケージに対するイベント．短縮できる．}
  \item{value}{関数又は関数のリスト，もしくは \code{action = "replace"}
    に対しては \code{NULL}．}
  \item{action}{取られるアクション．短縮できる．}
}

\details{
  \code{setHook} はユーザが，システム(又はユーザ)関数から呼び出される一連の関数である
  フックを登録する一般的な機構を提供する．
  フックの初期セットはパッケージや名前空間に関するイベントに
  関連していた：これらのフックは \code{packageEvent} への呼び出しで
  名前が付けられている．

  フックを完全に取り去るには \code{setHook(hookName, NULL, "replace")} を呼び出す．

  \R のパッケージが \code{\link{library}} で付加されるか他の手段でロードされる時，
  それは初期化コードを呼び出すことが出来る．
  初期化の途中で呼び出されるパッケージのフック関数に関する説明は
  \code{\link{.onLoad}} を見よ．
  ユーザは \code{setHook()} で提供されるフックを使って自分自身の初期化コードを
  加えることができる．これは呼び出し \code{\link{try}} の内部で
  \code{funname(pkgname, pkgpath)} として呼び出される関数である．

  イベントの系列はどのフックが定義されているかと，
  パッケージが付加されているのか単にロードされているのかに依存する．
  全てのフックが定義されパッケージが付加される場合には，初期化のイベントの
  順序は次のようになる：
  \enumerate{
  \item パッケージの名前空間がロードされる．
  \item パッケージの \code{\link{.onLoad}} 関数が実行される．
  \item もし S4 メソッドの選択適用がオンならば，
  \code{\link{setLoadAction}} で設定された任意のアクションが実行される．
  \item 名前空間が封印される．
  \item ユーザの \code{"onLoad"} フックが実行される．
  \item パッケージが検索パスに加えられる．
  \item パッケージの \code{\link{.onAttach}} 関数が実行される．
  \item パッケージの環境が封印される．
  \item ユーザの \code{"attach"} フックが実行される．
  }

  似たような系列(しかし逆向き)がパッケージが取り除かれその名前空間がアンロード
  されるときに実行される：
  \enumerate{
  \item ユーザの \code{"detach"} フックが実行される．
  \item パッケージの \code{\link{.Last.lib}} 関数が実行される．
  \item パッケージが検索パスから除かれる．
  \item ユーザの \code{"onUnload"} フックがが実行される．
  \item パッケージの \code{\link{.onUnload}} 関数が実行される．
  \item パッケージの名前区間がアンロードされる．
  }
  \R セッションが終了した時にはパッケージは除かれず名前空間はアンロードされないので，
  対応するフックは実行されないことを注意しよう．

  またユーザのフックはパッケージが検索パス上に無くても実行されるので，例におけるように
  パッケージ中のそれらのフックは二重(又は三重)コロン演算子で参照される必要があることを
  注意しよう．

  もし複数のフックが加えられると，それらは普通 \code{getHook} で示される順序で
  実行されるが，\code{"detach"} と \code{"onUnload"} フックは逆順で
  実行されるのでパッケージのイベントに対する既定はフックを既存のものの\sQuote{内部}
  に加えることである．

  フックは基本パッケージ中の \code{.userHooksEnv} 環境に\sQuote{複雑な}名前で保管される．
}

\value{
  \code{getHook} 関数に対しては関数のリスト(空かもしれない)．
  \code{setHook} 関数に対しては返り値はない．
  \code{packageEvent} に対しては導出されたフック名(文字列)．
}

\note{
  フックはそれらが修正するイベントの前に設定される必要がある：
  \pkg{methods} は起動列の初期に付加され現在 \pkg{utils} の名前空間を
  ロードするため，標準のパッケージに対してはこれは問題無しとはしない．
  下の例のようにフックを設定する普通の場所は
  \file{.Rprofile} ファイル中であるが，これは \pkg{methods} と \pkg{utils}
  に対しては上手く行かない．
}

\seealso{
  \code{\link{library}}, \code{\link{detach}}, \code{\link{loadNamespace}}.

  二重，三重コロン演算子についての議論は \code{\link{::}} を見よ．

  他のフックを後から加えることが出来る：関数 \code{\link{plot.new}} と
  \code{\link{persp}} は既にそれらを持つ．
}
\examples{
setHook(packageEvent("grDevices", "onLoad"),
        function(...) grDevices::ps.options(horizontal = FALSE))
}
\keyword{utilities}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/userhooks.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{userhooks}
%\alias{getHook}
%\alias{setHook}
%\alias{packageEvent}
%\alias{.userHooksEnv}
%\title{Functions to Get and Set Hooks for Load, Attach, Detach and Unload}
%\description{
%  These functions allow users to set actions to be taken before packages
%  are attached/detached and namespaces are (un)loaded.
%}
%\usage{
%getHook(hookName)
%setHook(hookName, value,
%        action = c("append", "prepend", "replace"))
%
%packageEvent(pkgname,
%             event = c("onLoad", "attach", "detach", "onUnload"))
%}
%\arguments{
%  \item{hookName}{character string: the hook name}
%  \item{pkgname}{character string: the package/namespace name}
%  \item{event}{character string: an event for the package.  Can be abbreviated.}
%  \item{value}{A function or a list of functions, or for \code{action = "replace"}, \code{NULL} }
%  \item{action}{The action to be taken.  Can be abbreviated.}
%}
%
%\details{
%  \code{setHook} provides a general mechanism for users to register
%  hooks, a list of functions to be called from system (or user)
%  functions.  The initial set of hooks was associated with events on
%  packages/namespaces: these hooks are named via calls to
%  \code{packageEvent}.
%
%  To remove a hook completely, call \code{setHook(hookName, NULL, "replace")}.
%
%  When an \R package is attached by \code{\link{library}} or loaded by
%  other means, it can call initialization code.  See
%  \code{\link{.onLoad}} for a description of the package hook functions
%  called during initialization.  Users can add their own initialization
%  code via the hooks provided by \code{setHook()}, functions which will
%  be called as \code{funname(pkgname, pkgpath)} inside a
%  \code{\link{try}} call.
%
%  The sequence of events depends on which hooks are defined, and whether
%  a package is attached or just loaded.  In the case where all hooks
%  are defined and a package is attached, the order of initialization
%  events is as follows:
%  \enumerate{
%  \item The package namespace is loaded.
%  \item The package's \code{\link{.onLoad}} function is run.
%  \item If S4 methods dispatch is on, any actions set by
%  \code{\link{setLoadAction}} are run.
%  \item The namespace is sealed.
%  \item The user's \code{"onLoad"} hook is run.
%  \item The package is added to the search path.
%  \item The package's \code{\link{.onAttach}} function is run.
%  \item The package environment is sealed.
%  \item The user's \code{"attach"} hook is run.
%  }
%
%  A similar sequence (but in reverse) is run when a package is detached
%  and its namespace unloaded:
%  \enumerate{
%  \item The user's \code{"detach"} hook is run.
%  \item The package's \code{\link{.Last.lib}} function is run.
%  \item The package is removed from the search path.
%  \item The user's \code{"onUnload"} hook is run.
%  \item The package's \code{\link{.onUnload}} function is run.
%  \item The package namespace is unloaded.
%  }
%  Note that when an \R session is finished, packages are not detached and
%  namespaces are not unloaded, so the corresponding hooks will not be
%  run.
%
%  Also note that some of the user hooks are run without the package
%  being on the search path, so in those hooks objects in the package
%  need to be referred to using the double (or triple) colon operator,
%  as in the example.
%
%  If multiple hooks are added, they are normally run in the order shown
%  by \code{getHook}, but the \code{"detach"} and \code{"onUnload"} hooks
%  are run in reverse order so the default for package events is to add
%  hooks \sQuote{inside} existing ones.
%
%  The hooks are stored in the environment \code{.userHooksEnv} in the
%  base package, with \sQuote{mangled} names.
%}
%
%\value{
%  For \code{getHook} function, a list of functions (possibly empty).
%  For \code{setHook} function, no return value.
%  For \code{packageEvent}, the derived hook name (a character string).
%}
%
%\note{
%  Hooks need to be set before the event they modify: for standard
%  packages this can be problematic as \pkg{methods} is loaded and
%  attached early in the startup sequence and currently that loads the
%  \pkg{utils} namepace.   The usual place to set hooks such as the
%  example below is in the \file{.Rprofile} file, but that will not work
%  for \pkg{methods} and \pkg{utils}.
%}
%
%\seealso{
%  \code{\link{library}}, \code{\link{detach}}, \code{\link{loadNamespace}}.
%
%  See \code{\link{::}}
%  for a discussion of the double and triple colon operators.
%
%  Other hooks may be added later: functions \code{\link{plot.new}} and
%  \code{\link{persp}} already have them.
%}
%\examples{
%setHook(packageEvent("grDevices", "onLoad"),
%        function(...) grDevices::ps.options(horizontal = FALSE))
%}
%\keyword{utilities}
%